
local ChatClient
local ChatConstants
local Logger
local Array
local PhotonTime
local tableutil
local byteutil
local photon

if pcall(require,"plugin.photon") then -- try to load Corona photon plugin
	print("Demo: ChatDemoClient module:","Corona plugin used")
    photon = require "plugin.photon"    
    ChatClient = photon.chat.ChatClient
    ChatConstants = photon.chat.constants
    Logger = photon.common.Logger
    Array = photon.common.type.Array
    PhotonTime = photon.common.util.time
    tableutil = photon.common.util.tableutil
    byteutil = photon.common.util.byteutil
else  -- or load photon.lua module
	print("Demo: ChatDemoClient module:","Lua lib used")
    photon = require("photon")
    ChatClient = require("photon.chat.ChatClient")
    ChatConstants = require("photon.chat.constants")
    Logger = require("photon.common.Logger")
    Array = require("photon.common.type.Array")
    PhotonTime = require("photon.common.util.time")
    tableutil = require("photon.common.util.tableutil")
    byteutil = require("photon.common.util.byteutil")
end

local DemoConstants =
{

    TickInterval = 0.500,

    LogLevel = Logger.Level.DEBUG,
}

local _P = function(x) print(tableutil.toStringReq(x)) end

math.randomseed( PhotonTime.timeFromStartMs() )

local appInfo = require("cloud-app-info")

local function createClient(view)
    local client = ChatClient.new(appInfo.NameServerAddress, appInfo.AppId, appInfo.AppVersion)

	client.view = view
    -- uncomment to use Custom Authentication
    client:setCustomAuthentication("username=" .. "yes" .. "&token=" .. "yes")

    client.logger:info("Init", appInfo.NameServerAddress, appInfo.AppId, appInfo.AppVersion)
    client.logger:setPrefix("Demo")
    client:setLogLevel(DemoConstants.LogLevel)
    
    function client:start()
--        self:connectToRegionFrontEnd("EU")
    end

    function client:onError(errorCode, errorMsg)
    	if errorCode == ChatClient.PeerErrorCode.FrontEndAuthenticationFailed then
    		errorMsg = errorMsg .. " with appId = " .. self.appId
    	end
        self.logger:error(errorCode, errorMsg)
		self.view:log("Error", errorCode, errorMsg)
    end
   
    function client:onStateChange(state)
		self.logger:info("State: ", state)
		self.view:onStateChange(state)
    end

	function client:onChatMessages(channelName, messages) 
		for _, m in pairs(messages) do
			local sender = m:getSender()
			if sender == client:getUserId() then
				sender = "me"
			end
			view:showMessage('[' .. channelName .. ':' .. sender .. ']', m:getContent())
		end
	end
	
	function client:onPrivateMessage(channelName, m) 
		local sender = m:getSender()
		if sender == client:getUserId() then
			sender = "me"
		end
		view:showMessage('[' .. channelName .. '@' .. sender .. ']', m:getContent())
	end
	
	function client:onUserStatusUpdate(userId, status, gotMessage, statusMessage) 
		local msg = tostring(statusMessage)
		if not gotMessage then
			msg = "[message skipped]"
		end
		view:showMessage( "[i]", userId .. ": " .. tostring(ChatConstants.UserStatusToName(status)) .. "(" .. tostring(status) .. ") / " .. msg)
	end

	function client:sendMessage(text)
		local chDelim = string.find(text, ":")
		local userDelim = string.find(text, "@")
		if chDelim then
			local ch, t = string.sub(text, 1, chDelim - 1), string.sub(text, chDelim + 1)
			self:publishMessage(ch, t)
			self.logger:info("publish: ", ch, t)
		elseif userDelim then
			local u, t = string.sub(text, 1, userDelim - 1), string.sub(text, userDelim + 1)
			self:sendPrivateMessage(u, t)
			self.logger:info("send private: ", u, t)
		else
			local exists = false
			for chName, _ in pairs(self:getPublicChannels()) do
				self:publishMessage(chName, text)
				self.logger:info("publish: ", chName, text)
				exists = true
				break
			end
			if not exists then
				view:showMessage("[i]", "error: no subscribed channels")
			end
		end
	end

	function client:onSubscribeResult(results)
		results = results or {}
		self.logger:info("onSubscribeResult", results)
		local m = "---- subscribed to "
		for k, v in pairs(results) do
			self.logger:info("    ", k, v)
			if v then
				m = m .. "'" .. k .. "', "
			end
		end
		view:showMessage("[i]", {
			m,
			"type in 'channel:message'",
			"   and press 'OK' to publish"
			})
		view:onSubscribed()
	end

	function client:onUnsubscribeResult(results)
		results = results or {}
		self.logger:info("onUnsubscribeResult", results)
		local m = "unsubscribed from "
		for k, v in pairs(results) do
			self.logger:info("    ", k, v)
			m = m .. "'" .. k .. "', "
		end
		view:showMessage("[i]", m)
		view:onUnsubscribed()
	end

	function client:onOperationResponse(errorCode, errorMsg, code, content) 
		if errorCode ~= 0 then
			view:showMessage("[i]", "error: " .. tostring(errorMsg) .. '(op ' .. code .. ')')
		end
	end 
	
	
	return client
end


return createClient