local Logger
local ChatClient
local ChatConstants
local tableutil
local PhotonTime
local photon

if pcall(require,"plugin.photon") then -- load photon plugin
    print("Demo: main module:","Corona plugin used")
    photon = require "plugin.photon"
    Logger = photon.common.Logger
    ChatClient = photon.chat.ChatClient
	ChatConstants = photon.chat.constants
    tableutil = photon.common.util.tableutil
    PhotonTime = photon.common.util.time
else  -- load photon.lua module
    print("Demo: main module:","Lua lib used")
    photon = require("photon")
    Logger = require("photon.common.Logger")
    ChatClient = require("photon.chat.ChatClient")
	ChatConstants = require("photon.chat.constants")
    tableutil = require("photon.common.util.tableutil")
    PhotonTime = require("photon.common.util.time")
end
local USER_COUNT = 10
local CHAT_LINE_COUNT = 5
local widget = require("widget")
local uihelper = require("uihelper")
local onScreenKeyboard = require("onScreenKeyboard")

local view = {}
local client = require("ChatDemoClient")(view)

local textSize1 = display.pixelHeight * 3 / 100
local textSize2 = display.pixelHeight * 5 / 100
--local inputSize = display.pixelHeight * 12 / 100
local buttonWidth = display.pixelWidth / 2 - 10
local buttonHeight = buttonWidth / 5

local function newButton(label, handler, autoButtonSize, fontSize) 
	fontSize = fontSize or textSize2
	local width, height = buttonWidth, buttonHeight
	if autoButtonSize then
		local tmp = display.newText(label, 0, 0, 0, 0, native.systemFont, fontSize)
		width = tmp.width
		height = tmp.height
		display.remove(tmp)
	end
	return widget.newButton{
    label = label,
	labelAlign = "center",
	fontSize = fontSize,
    width = width, height = height,
    onEvent = 
        function(event) 
            if "ended" == event.phase then
				handler()
            end
        end       
}
end

-- context dependent buttons
local subscribeButton
local setFriendsButton
local subscribeState
local setFriendsState
local function setSubscribeState(s)
	if subscribeState ~= s then
		local t = {"Subscribe", "Unsubscr."}
		subscribeButton:setLabel(t[s + 1])
		subscribeState = s
	end
end
local function setSetFriendsState(s)
	if setFriendsState ~= s then
		local t = {"Set friends", "Clr. friends"}
		setFriendsButton:setLabel(t[s + 1])
		setFriendsState = s
	end
end

local stateText = display.newText("undef", 0, 0, 0, 0, native.systemFont, textSize1)
local userIdInput = display.newText("user" .. math.floor(math.random() * USER_COUNT),  0, 0, buttonWidth, 0, native.systemFont, textSize2)
local chatText = {}
for i = 1, CHAT_LINE_COUNT do 
	chatText[i] = display.newText("",  0, 0, display.contentWidth * 10, 0, native.systemFont, textSize1) -- no wrap (lost text)
end
local messageInput = display.newText("",  0, 0, display.contentWidth, 0, native.systemFont, textSize2)

display.setStatusBar(display.HiddenStatusBar)
local scene = display.newGroup()
local panel = uihelper.newVertPanel()
panel:add( scene )
panel:add( stateText )
panel:addNewLine()
panel:add( display.newText("User id:",  0, 0, buttonWidth, 0, native.systemFont, textSize2) )
panel:add( userIdInput )
panel:add( newButton("Connect", function()
		client:setUserId(userIdInput.text)
		client:connectToRegionFrontEnd("EU")
	end) )	
panel:add( newButton("Disconnect", function()
		client:disconnect()
	end) )
subscribeButton = newButton("(subscr)", function()
	if subscribeState == 0 then
		if client:subscribe({"a", "b", "c"}) then
			view:showMessage("[i]", "subscribing...")
		else
			view:showMessage("[i]", {
				"error: subscribe send failed",  
				"   [Connect] first?"
				})
		end
	else
		if (client:unsubscribe({"a", "b", "c", "z"})) then
			view:showMessage("[i]", "unsubscribing...")
		end
	end
end) 
panel:add(subscribeButton)
setSubscribeState(0)

setFriendsButton = newButton("(friends)", function()
	local friends = {}
	for i = 0, USER_COUNT - 1 do
		table.insert(friends, "user" .. i)
	end
	if setFriendsState == 0 then
		if client:addFriends(friends) then
			view:showMessage("[i]", "adding friends: " .. table.concat(friends, ",") .. "...")
			setSetFriendsState(1)
		else
			view:showMessage("[i]", {
				"error: set friends send failed",  
				"   [Connect] first?"
				})		
		end
	else
		if client:removeFriends(friends) then
			view:showMessage("[i]", "removing friends: " .. table.concat(friends, ",") .. "...")
			setSetFriendsState(0)
		end
	end
end)
panel:add(setFriendsButton)
setSetFriendsState(0)

for i = 1, #chatText do
	panel:add( chatText[i] )
end
local s = ChatConstants.UserStatus.Offline
while true do
	local n = ChatConstants.UserStatusToName(s)
	if not n then break end
	
	local s2 = s
	panel:add( newButton(n, function()			
			if not client:setUserStatus(s2, "hey, i'm " .. n, true) then -- update message
			--if not client:setUserStatus(s2, nil, true) then -- skip message
			--if not client:setUserStatus(s2) then -- clear message
				view:showMessage("[i]", {
					"error: status send failed",
					"   [Connect] first?"
					})
			else
				view:showMessage("[i]", "my status sent: " .. n)
			end
		end
		, true, textSize1 ))
	s = s + 1
end

panel:add( messageInput )

local dummy = display.newGroup()
panel:add(dummy)

local disconnected = true
local keyboard = onScreenKeyboard:new({START_Y = dummy.y, IGNORE_ORIENTATION_CHANGES = true}) 
keyboard:setListener(
	function(event)
		if(event.phase == "ended")  then
			if disconnected then 
				userIdInput.text=keyboard:getText()
			elseif client:isConnectedToFrontEnd() then
				messageInput.text=keyboard:getText()
			end
			--check whether the user finished writing with the keyboard. The inputCompleted
			--flag of  the keyboard is set to true when the user touched its "OK" button
			if(event.target.inputCompleted == true) then
				if client:isConnectedToFrontEnd() then
					client:sendMessage(keyboard.text)
					keyboard.text = ""                                        
					messageInput.text = ""
				end
			end
		end
	end
)
keyboard:drawKeyBoard(keyboard.keyBoardMode.letters_small)

function view:onStateChange(state, info)
    stateText.text = "Photon Chat. " .. tostring(ChatClient.StateToName(state))
    if info then
        stateText.text = stateText.text .. " " .. info
    end
	if state == ChatClient.State.ConnectedToFrontEnd then
		view:showMessage("[i]", {
			"---- connected to Front End",
			"[Subscribe] for public channels",
			"   or type in 'userid@message'",
			"   and press 'OK' for private"
			})
		setSubscribeState(0)
	end
	disconnected = state == ChatClient.State.Uninitialized or state == ChatClient.State.Disconnected
	if disconnected then
		view:showMessage("[i]", "type in user id and press [Connect]")
		setSubscribeState(0)
		setSetFriendsState(0)
	end
	
	-- clear input on 'focus' change
	keyboard.text = ""                                        
	messageInput.text = ""

end

function view:log(...)
    view:showMessage("[i]", client.logger:format( ... ))
end

function view:showMessage(source, msg)
	if type(msg) == 'table' then
		for _, m in pairs(msg) do
			view:showMessage(source, m)
		end
	else 
		for i = 2, #chatText do
			chatText[i - 1].text = chatText[i].text
		end
		chatText[#chatText].text = tostring(source) .. ": " .. tostring(msg)
	end
end

function view:onSubscribed()
	setSubscribeState(1)
end

function view:onUnsubscribed()
	setSubscribeState(0)
end

view:showMessage("[i]", "Welcome")
view:onStateChange(ChatClient.State.Uninitialized)
client:start()
timer.performWithDelay( 100, function() client:service() end, 0)
